@@ -1,6 +1,7 @@
 rbind.zoo <- function(..., deparse.level = 1)
 {  
-  args <- list(...)
+
+  args <- Filter(Negate(is.null), list(...))
   indexes <- do.call("c", lapply(args, index))
 
   my.table <- function(x) {
@@ -9,6 +10,9 @@ rbind.zoo <- function(..., deparse.level = 1)
   }
   if(max(my.table(indexes)) > 1) stop("indexes overlap")
 
+  if(any(sapply(args, function(x) is.null(dim(x)) && length(x) == 0 && length(index(x)) > 0)))
+    stop("zero-length vectors with non-zero-length index are not allowed")  
+
   ncols <- sapply(args, NCOL)  
   if(!all(ncols == ncols[1])) stop("number of columns differ")
 
@@ -30,7 +34,7 @@ rbind.zoo <- function(..., deparse.level = 1)
   }
   if(!namsOK) warning("column names differ")
 
-  if(ncols[1] > 1)
+  if((ncols[1] > 1) | !all(sapply(args, function(a) is.null(dim(a)))))
     rval <- zoo(do.call("rbind", lapply(args, coredata)), indexes)
   else
     rval <- zoo(do.call("c", lapply(args, coredata)), indexes)
@@ -66,10 +70,15 @@ merge.zoo <- function(..., all = TRUE, fill = NA, suffixes = NULL, retclass = c(
     cl[[1]] <- cl$all <- cl$fill <- cl$retclass <- cl$suffixes <- NULL
     args <- list(...)
 
+	# remove NULL args
+	isnull <- sapply(args, is.null)
+	cl <- cl[!isnull]
+	args <- args[!isnull]
+
     parent <- parent.frame()
 
     is.plain <- function(x) 
-	all(class(x) %in% c("array", "integer", "numeric", "factor", "matrix"))
+	all(class(x) %in% c("array", "integer", "numeric", "factor", "matrix", "logical"))
 
     is.scalar <- function(x) is.plain(x) && length(x) == 1
 
@@ -80,7 +89,7 @@ merge.zoo <- function(..., all = TRUE, fill = NA, suffixes = NULL, retclass = c(
     scalars <- sapply(args, is.scalar)
 
     if(!is.zoo(args[[1]])) args[[1]] <- as.zoo(args[[1]])
-    for(i in seq(along = args))
+    for(i in seq_along(args))
         if (is.plain(args[[i]]))  
             args[[i]] <- zoo(args[[i]], index(args[[1]]), attr(args[[1]], "frequency"))
 	else if (!is.zoo(args[[i]]))
@@ -101,7 +110,7 @@ merge.zoo <- function(..., all = TRUE, fill = NA, suffixes = NULL, retclass = c(
         makeNames <- function(l) {
             nm <- names(l)
             fixup <- if (is.null(nm)) 
-                seq(along = l)
+                seq_along(l)
             else nm == ""
             dep <- sapply(l[fixup], function(x) deparse(x)[1])
             if (is.null(nm)) 
@@ -135,7 +144,7 @@ merge.zoo <- function(..., all = TRUE, fill = NA, suffixes = NULL, retclass = c(
     # fn to get the unique elements in x, in sorted order, using only
     # [, match, length and order
     sort.unique <- function(x) {
-        x <- x[MATCH(x, x) == seq(length = length(x))]
+        x <- x[MATCH(x, x) == seq_len(length(x))]
         x[ORDER(x)]
     }
 
@@ -314,11 +323,11 @@ merge.zoo <- function(..., all = TRUE, fill = NA, suffixes = NULL, retclass = c(
             rval[rval == ""] <- suffixes[i]
             rval
         }
-        zoocolnames <- lapply(seq(along = args), f)
+        zoocolnames <- lapply(seq_along(args), f)
         f <- function(i) ifelse(fixme[[i]], paste(zoocolnames[[i]], 
             suffixes[i], sep = "."), zoocolnames[[i]])
         if (any(duplicated(unlist(zoocolnames)))) 
-            zoocolnames <- lapply(seq(along = args), f)
+            zoocolnames <- lapply(seq_along(args), f)
         colnames(rval) <- make.unique(unlist(zoocolnames))
     } else {
         fixcolnames <- function(a) {
@@ -329,7 +338,7 @@ merge.zoo <- function(..., all = TRUE, fill = NA, suffixes = NULL, retclass = c(
             else return(paste(".", 1:NCOL(a), sep = ""))
         }
         zoocolnames <- lapply(args, fixcolnames)
-        zoocolnames <- unlist(lapply(seq(along = args), function(i) 
+        zoocolnames <- unlist(lapply(seq_along(args), function(i) 
 		if (!is.null(zoocolnames[[i]])) # NULL returned if false
 			paste(suffixes[i], zoocolnames[[i]], sep = ""))
 	)
