@@ -1,5 +1,6 @@
 read.zoo <- function(file, format = "", tz = "", FUN = NULL,
-  regular = FALSE, index.column = 1, aggregate = FALSE, ...)
+  regular = FALSE, index.column = 1, drop = TRUE, FUN2 = NULL, 
+  split = NULL, aggregate = FALSE, ...)
 {
   ## `file' and `...' is simply passed to read.table
   ## the first column is interpreted to be the index, the rest the coredata
@@ -7,12 +8,12 @@ read.zoo <- function(file, format = "", tz = "", FUN = NULL,
   ## defaults for `FUN' are guessed and are numeric, Date or POSIXct
 
   ## read data
-  rval <- read.table(file, ...)
+  rval <- if (is.data.frame(file)) file else read.table(file, ...)
 
   ## if `file' does not contain data
   if(NROW(rval) < 1) {
     if(is.data.frame(rval)) rval <- as.matrix(rval)
-    if(NCOL(rval) > 1) rval <- rval[,-index.column]
+    if(NCOL(rval) > 1) rval <- rval[,-index.column, drop = drop]
     rval <- zoo(rval)
     return(rval)
   }
@@ -21,10 +22,20 @@ read.zoo <- function(file, format = "", tz = "", FUN = NULL,
   if(NCOL(rval) < 1) stop("data file must specify at least one column")
   
   ## extract index, retain rest of the data
-  if (NCOL(rval) == 1) ix <- seq(length = NROW(rval))
+  if (NCOL(rval) == 1) ix <- seq_len(NROW(rval))
   else {
     ix <- rval[,index.column]
-    rval <- rval[,-index.column]
+	split.values <- if (!is.null(split)) {
+		if (is.finite(split)) rval[, split]
+		else {
+			s <- split
+			split <- 0
+			if (s == Inf) ave(ix, ix, FUN = seq_along)
+			else if (s == -Inf) ave(ix, ix, FUN = function(x) rev(seq_along(x)))
+			else ix
+		}
+	}
+    rval <- rval[,-c(split, index.column), drop = drop]
   }
   if(is.factor(ix)) ix <- as.character(ix)
   if(is.data.frame(rval)) rval <- as.matrix(rval)
@@ -61,6 +72,7 @@ read.zoo <- function(file, format = "", tz = "", FUN = NULL,
         else if (is.numeric(ix)) toNumeric
         else toDefault
   }
+  FUN <- match.fun(FUN)
   
   ## compute index from (former) first column
   ix <- if (missing(format)) {
@@ -69,9 +81,23 @@ read.zoo <- function(file, format = "", tz = "", FUN = NULL,
     if (missing(tz)) FUN(ix, format = format) 
     else FUN(ix, format = format, tz = tz)
   }
+
+  if (!is.null(FUN2)) {
+	FUN2 <- match.fun(FUN2)
+	ix <- FUN2(ix)
+  }
   
   ## sanity checking
-  if(any(is.na(ix))) stop("index contains NAs")
+  if(any(is.na(ix))) {
+    idx <- which(is.na(ix))
+	msg <- if (length(idx) == 1)
+		paste("index has bad entry at data row", idx)
+	else if (length(idx) <= 100)
+		paste("index has bad entries at data rows:", paste(idx, collapse = " "))
+	else paste("index has", length(idx), "bad entries at data rows:", 
+		paste(head(idx, 100), collapse = " "), "...")
+	stop(msg)
+  }
   if(length(ix) != NROW(rval)) stop("index does not match data")
   
   ## setup zoo object and return 
@@ -85,15 +111,24 @@ read.zoo <- function(file, format = "", tz = "", FUN = NULL,
     if(!is.function(agg.fun)) stop(paste("invalid specification of", sQuote("aggregate")))
   }
   remove(list = "aggregate")
-  withCallingHandlers(rval <- zoo(rval, ix), warning = 
-    function(w) {
-        if (!is.null(agg.fun) && !is.na(pmatch("some methods for", w$message)))
-            invokeRestart("muffleWarning")
-    }
-  )
 
-  if(regular && is.regular(rval)) rval <- as.zooreg(rval)
-  if (!is.null(agg.fun)) rval <- aggregate(rval, time(rval), agg.fun)
+  if (is.null(split)) {
+	rval <- if (!is.null(agg.fun)) aggregate(zoo(rval), ix, agg.fun)
+	else zoo(rval, ix)
+    if(regular && is.regular(rval)) rval <- as.zooreg(rval)
+  } else {
+	split.matrix <- split.data.frame
+	rval <- split(rval, split.values)
+	ix <- split(ix, split.values)
+	rval <- mapply(zoo, rval, ix)
+    if(regular) {
+		rval <- lapply(rval, function(x) if (is.regular(x)) as.zooreg(x) else x)
+	}
+	if (!is.null(agg.fun)) rval <-
+		lapply(seq_along(rval), function(z) aggregate(z, time(z), agg.fun))
+	rval <- do.call(merge, rval)
+  }
+	
   return(rval)
 }
 
