@@ -1,7 +1,7 @@
-window.zoo <- function(x, index = index.zoo(x), start = NULL, end = NULL, ...)
+window.zoo <- function(x, index. = index(x), start = NULL, end = NULL, ...)
 {
-  all.indexes <- index.zoo(x)
-  in.index <- MATCH(all.indexes, index, nomatch = 0) > 0
+  all.indexes <- index(x)
+  in.index <- MATCH(all.indexes, index., nomatch = 0) > 0
 
   if(length(start) == 2 && !is.null(attr(x, "frequency")) && is.numeric(all.indexes)) {
     freq <- attr(x, "frequency")
@@ -14,7 +14,7 @@ window.zoo <- function(x, index = index.zoo(x), start = NULL, end = NULL, ...)
 
   if(is.null(start)) {
     if(is.null(end)) {
-      wi <- which(MATCH(all.indexes, index, nomatch = 0) > 0)
+      wi <- which(MATCH(all.indexes, index., nomatch = 0) > 0)
       return(x[wi,,])
     } else {
       wi <- which(in.index & all.indexes <= end)
@@ -30,10 +30,10 @@ window.zoo <- function(x, index = index.zoo(x), start = NULL, end = NULL, ...)
   }
 }
 
-"window<-.zoo" <- function(x, index = index.zoo(x), start = NULL, end = NULL, ..., value)
+"window<-.zoo" <- function(x, index. = index(x), start = NULL, end = NULL, ..., value)
 {
-  ix <- index.zoo(x)
-  stopifnot(all(MATCH(index, ix, nomatch = 0) > 0))
+  ix <- index(x)
+  stopifnot(all(MATCH(index., ix, nomatch = 0) > 0))
   
   if(length(start) == 2 && !is.null(attr(x, "frequency")) && is.numeric(ix)) {
     freq <- attr(x, "frequency")
@@ -44,10 +44,10 @@ window.zoo <- function(x, index = index.zoo(x), start = NULL, end = NULL, ...)
     end <- floor(end[1]*freq + (end[2] - 1) + .0001)/freq
   }
   
-  if (!is.null(start)) index <- index[index >= start]
-  if (!is.null(end)) index <- index[index <= end]
+  if (!is.null(start)) index. <- index.[index. >= start]
+  if (!is.null(end)) index. <- index.[index. <= end]
 
-  wi <- which(MATCH(ix, index, nomatch = 0) > 0)
+  wi <- which(MATCH(ix, index., nomatch = 0) > 0)
   if (length(dim(x)) == 0)
   	  x[wi] <- value
   else
@@ -80,12 +80,13 @@ lag.zoo <- function(x, k = 1, na.pad = FALSE, ...)
 
 
 
-lag.zooreg <- function(x, k = 1, ...)
+lag.zooreg <- function(x, k = 1, na.pad = FALSE, ...)
 {
    if (length(k) > 1) {
 	if (is.null(names(k))) names(k) <- paste("lag", k, sep = "")
-	return(do.call("merge.zoo", lapply(k, lag.zoo, x = x, ...)))
+	return(do.call("merge.zoo", lapply(k, lag.zooreg, x = x, na.pad = na.pad, ...)))
    }
+   x0 <- x
    nr <- NROW(x)
    freq <- attr(x, "frequency")
    
@@ -96,8 +97,8 @@ lag.zooreg <- function(x, k = 1, ...)
    ix <- if(identical(class(ix), "numeric") | identical(class(ix), "integer"))
      floor(freq*ix - k + .0001)/freq else ix - k/freq
    index(x) <- ix
-   
-   return(x)
+
+   if (na.pad) merge(x, zoo(, time(x0))) else x
 }
 
 diff.zoo <- function(x, lag = 1, differences = 1, arithmetic = TRUE, na.pad = FALSE, ...)
